@@ -9,7 +9,6 @@ use {
         packet::{to_packet_batches, PacketBatch},
         sigverify,
     },
-    std::time::Duration,
     test::Bencher,
 };
 
@@ -24,13 +23,13 @@ fn test_packet_with_size(size: usize, rng: &mut ThreadRng) -> Vec<u8> {
 
 fn do_bench_dedup_packets(bencher: &mut Bencher, mut batches: Vec<PacketBatch>) {
     // verify packets
-    let mut deduper = sigverify::Deduper::new(1_000_000, Duration::from_millis(2_000));
+    let mut deduper = sigverify::Deduper::new(1_000_000, 2_000);
     bencher.iter(|| {
         let _ans = deduper.dedup_packets(&mut batches);
         deduper.reset();
         batches
             .iter_mut()
-            .for_each(|b| b.packets.iter_mut().for_each(|p| p.meta.discard = false));
+            .for_each(|b| b.packets.iter_mut().for_each(|p| p.meta.set_discard(false)));
     });
 }
 
@@ -112,7 +111,7 @@ fn bench_dedup_baseline(bencher: &mut Bencher) {
 #[bench]
 #[ignore]
 fn bench_dedup_reset(bencher: &mut Bencher) {
-    let mut deduper = sigverify::Deduper::new(1_000_000, Duration::from_millis(0));
+    let mut deduper = sigverify::Deduper::new(1_000_000, 0);
     bencher.iter(|| {
         deduper.reset();
     });
